// Connect.cpp : Implementation of CConnect
#include "stdafx.h"
#include "AddIn.h"
#include "Connect.h"

#define INITGUID
#include "Addin_i.c"

#include <atlenc.h>
#include <vector>

CString _clip_format;
std::vector<CHAR> _shape_data;


void CConnect::SaveShape(Visio::IVShapePtr shape)
{
	IDataObjectPtr data_obj;
	shape->QueryInterface(__uuidof(IDataObject), (void**)&data_obj);

	COleDataObject ole_data_obj;
	ole_data_obj.Attach(data_obj, FALSE);

	FORMATETC fmt = {0};

	ole_data_obj.BeginEnumFormats();
	while (ole_data_obj.GetNextFormat(&fmt))
	{
		WCHAR clipformat_name[1024] = L"";
		GetClipboardFormatName(fmt.cfFormat, clipformat_name, 1024);

		if (StrStrIW(clipformat_name, L"Shapes"))
		{
			STGMEDIUM stg = {0};
			if (ole_data_obj.GetData(fmt.cfFormat, &stg, &fmt))
			{
				if (stg.tymed == TYMED_HGLOBAL)
				{
					int size = GlobalSize(stg.hGlobal);
					LPBYTE data = (LPBYTE)GlobalLock(stg.hGlobal);

					_clip_format = clipformat_name;

					int result_size = Base64EncodeGetRequiredLength(size);
					_shape_data.resize(result_size);
					Base64Encode(data, size, &_shape_data[0], &result_size, 0);
				}
			}
		}
	}
}



void CConnect::DropSavedShape(Visio::IVPagePtr page)
{
	COleDataSource ds;

	CLIPFORMAT cf = RegisterClipboardFormat(_clip_format);

	int result_size = Base64DecodeGetRequiredLength(_shape_data.size());

	HGLOBAL hBuffer = GlobalAlloc (GMEM_MOVEABLE|GMEM_SHARE, result_size);
	LPBYTE data = (LPBYTE)GlobalLock (hBuffer);
	Base64Decode(&_shape_data[0], _shape_data.size(), data, &result_size);
	GlobalUnlock(hBuffer);

	ds.CacheGlobalData(cf, hBuffer);

	IDataObjectPtr data_obj = ds.GetInterface(&__uuidof(IDataObject));
	page->Drop(data_obj, 0, 0);
}



void __stdcall CConnect::button_SaveToStream(IDispatch *ctrl, VARIANT_BOOL *cancel_default)
{
	Visio::IVWindowPtr activeWindow = _application->ActiveWindow;
	if (activeWindow == NULL)
		return;

	Visio::IVSelectionPtr selection = activeWindow->Selection;
	if (selection == NULL || selection->Count == 0)
		return;

	SaveShape(selection->Item[1L]);

	::MessageBox(NULL, L"Shape saved", L"", 0);
}



void __stdcall CConnect::button_PasteFromStream(IDispatch *ctrl, VARIANT_BOOL *cancel_default)
{
	Visio::IVPagePtr page = _application->ActivePage;
	if (page == NULL)
		return;

	DropSavedShape(page);

	::MessageBox(NULL, L"Shape pasted", L"", 0);
}



BEGIN_OBJECT_MAP(ObjectMap)
	OBJECT_ENTRY(__uuidof(Connect), CConnect)
END_OBJECT_MAP()

CComModule _Module;
class CAddinApp : public CWinApp 
{
	virtual BOOL InitInstance()
	{
		AFX_MANAGE_STATE(AfxGetStaticModuleState());

	_Module.SetResourceInstance(AfxGetResourceHandle());

		// Initialize COM stuff
		if (FAILED(_Module.Init(ObjectMap, AfxGetInstanceHandle(), &LIBID_ShapePersistenceLib)))
			return FALSE;

		return TRUE;
	}

	virtual int ExitInstance()
	{
		AFX_MANAGE_STATE(AfxGetStaticModuleState());

		_Module.Term();

		return 0;
	}
};

CAddinApp theApp;

Office::_CommandBarButtonPtr CConnect::AddButton(Office::CommandBarPtr cb, LPCWSTR caption, long id)
{
	Office::_CommandBarButtonPtr button =
		cb->Controls->Add(Office::msoControlButton, vtMissing, vtMissing, vtMissing, true);

	button->Tag = caption;
	button->Caption = caption;
	button->FaceId = id;

	return button;
}

void CConnect::InstallToolbar() 
{
	IDispatchPtr cbs_disp;
	_application->get_CommandBars(&cbs_disp);
	Office::_CommandBarsPtr cbs = cbs_disp;

	Office::CommandBarPtr cb = cbs->Add("Bar", vtMissing, vtMissing, true);
	cb->Visible = true;

	_buttonSaveToStream = AddButton(cb, L"SaveToStream C++", 134);
	IDispEventSimpleImpl<1, CConnect, &__uuidof(Office::_CommandBarButtonEvents)>::DispEventAdvise(_buttonSaveToStream);

	_buttonPasteFromStream = AddButton(cb, L"PasteFromStream C++", 135);
	IDispEventSimpleImpl<2, CConnect, &__uuidof(Office::_CommandBarButtonEvents)>::DispEventAdvise(_buttonPasteFromStream);
}

STDMETHODIMP CConnect::OnConnection(IDispatch *pApplication, AddInDesignerObjects::ext_ConnectMode /*ConnectMode*/, IDispatch *pAddInInst, SAFEARRAY ** /*custom*/ )
{
	pApplication->QueryInterface(__uuidof(IDispatch), (LPVOID*)&_application);
	pAddInInst->QueryInterface(__uuidof(IDispatch), (LPVOID*)&_addInInstance);

	InstallToolbar();

	return S_OK;
}

STDMETHODIMP CConnect::OnDisconnection(AddInDesignerObjects::ext_DisconnectMode /*RemoveMode*/, SAFEARRAY ** /*custom*/ )
{
	_application = NULL;
	_addInInstance = NULL;
	return S_OK;
}

STDMETHODIMP CConnect::OnAddInsUpdate (SAFEARRAY ** /*custom*/ )
{
	return S_OK;
}

STDMETHODIMP CConnect::OnStartupComplete (SAFEARRAY ** /*custom*/ )
{
	return S_OK;
}

STDMETHODIMP CConnect::OnBeginShutdown (SAFEARRAY ** /*custom*/ )
{
	return S_OK;
}
